// ======================================================================
// ePost.cpp - main program of an English Part-of-Speech Tagger
// 
// 102300: Benjamin Han <benhdj@cs.cmu.edu> Created.
// 110500: Benjamin Han <benhdj@cs.cmu.edu> Added a simple exception for
//         rudimentary file IO error handling.
// 080401: Benjamin Han <benhdj@cs.cmu.edu> Adapted for MSVC++; chaned 
//         license to LGPL.
// ======================================================================

//   Copyright (C) 2001 Benjamin Han <benhdj@cs.cmu.edu>
//   The original Transformation-based Part-of-Speech Tagger is written
//   by Eric Brill <brill@cs.jhu.edu> - see the separate copyright notice
//   under post directory
//
//    This library is free software; you can redistribute it and/or
//    modify it under the terms of the GNU Lesser General Public
//    License as published by the Free Software Foundation; either
//    version 2.1 of the License, or (at your option) any later version.
//
//    This library is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//    Lesser General Public License for more details.
//
//    You should have received a copy of the GNU Lesser General Public
//    License along with this library; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include "post.h"
#include "except.h"

#include <string>
#include <iostream>
#include <list>

#ifdef __GNUG__
#include <fstream>
#else
#define BUF_SIZE 10240
#endif

using namespace std;

#define ERR_SUCCESS                    0
#define ERR_ILLEGAL_OPT                1
#define ERR_OPEN_INPUT_FILE            2
#define ERR_ILLEGAL_CORPUS_FILE_FORMAT 3
#define ERR_POSTERROR                  4

void printHelp ()
{
  cerr<<"* epost input-file"<<endl<<endl;
  cerr<<"  input-file: A formatted and tokenized English corpus."<<endl<<endl;  
  cerr<<"  *IMPORTANT*: DO NOT INCLUDE EMPTY LINES OR LINES WITH ONLY WHITE CHARS!"<<endl<<endl;
}

int main(int argc, char* argv[])
{
#ifdef __GNUG__
  ifstream ifs;
#else
  FILE *fp;
  char buf[BUF_SIZE];
#endif
  list<string> sList;
  string s;

  cerr<<"* Brill's English Part-of-Speech Tagger v1.0, adpated by Benjamin Han "<<endl<<endl;

  if (argc!=2)
    {
      printHelp();
      cerr<<"* Illegal command-line syntax."<<endl;
      return ERR_ILLEGAL_OPT;
    }

#ifdef __GNUG__
  // for some mysterious reason VC++ kept spitting "unresolved symbol" on this C++ code
  // that's why the file reading code is changed to the stone-age version
  ifs.open(argv[1]);
  if (!ifs)
    {
      cerr<<"* ERROR: Cannot open "<<argv[1]<<" for reading."<<endl;
      return ERR_OPEN_INPUT_FILE;
    }

  while (getline(ifs,s)) sList.push_back(s);
  ifs.close();
#else
  fp=fopen(argv[1],"r");
  if (!fp)
  {
    cerr<<"* ERROR: Cannot open "<<argv[1]<<" for reading."<<endl;
    return ERR_OPEN_INPUT_FILE;
  }

  while (fgets(buf,BUF_SIZE,fp))
  {
	s=buf;
	if (s[s.length()-1]='\n') s.erase(s.length()-1);
	sList.push_back(s);
  }
  fclose(fp);
#endif

    try
    {
      POST post;
      list<string> sListBackup;
      
      sListBackup=sList;
      for (int i=0;i<10;++i) {
	sList=sListBackup;
	post.run(sList);
	copy(sList.begin(),sList.end(),ostream_iterator<string>(cout,"\n"));	
	cout<<"----------------------------------------------------------------------"<<endl;	
      }
    }

  catch (Exception &e)
    {
      cerr<<"* ERROR: "<<e<<endl;
      return ERR_POSTERROR;
    }
  
  return ERR_SUCCESS;
}
