// ======================================================================
// post.cpp - an English Part-of-Speech Tagger based on Eric Brill's code
// 
// 102300: Benjamin Han <benhdj@cs.cmu.edu> Created.
// 110500: Benjamin Han <benhdj@cs.cmu.edu> Added a simple exception for
//         rudimentary file IO error handling.
// 080401: Benjamin Han <benhdj@cs.cmu.edu> Adapted for MSVC++; chaned 
//         license to LGPL.
// ======================================================================

//   Copyright (C) 2001 Benjamin Han <benhdj@cs.cmu.edu>
//   The original Transformation-based Part-of-Speech Tagger is written
//   by Eric Brill <brill@cs.jhu.edu> - see the separate copyright notice
//   under post directory
//
//    This library is free software; you can redistribute it and/or
//    modify it under the terms of the GNU Lesser General Public
//    License as published by the Free Software Foundation; either
//    version 2.1 of the License, or (at your option) any later version.
//
//    This library is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//    Lesser General Public License for more details.
//
//    You should have received a copy of the GNU Lesser General Public
//    License along with this library; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include "post.h"

POST::POST ()
{
  FILE *lexicon, *bigrams, *lRuleFile, *cRuleFile;
  int i=0;

  lexicon = fopen(LEXICON,"r");
  bigrams = fopen(BIGRAMS,"r");
  lRuleFile = fopen(LRULEFILE,"r");
  cRuleFile = fopen(CRULEFILE,"r");

  if (lexicon==NULL) throw (POSTFileError(LEXICON));
  if (bigrams==NULL) throw (POSTFileError(BIGRAMS));
  if (lRuleFile==NULL) throw (POSTFileError(LRULEFILE));
  if (cRuleFile==NULL) throw (POSTFileError(CRULEFILE));

  Tagger (lexicon, bigrams, lRuleFile, cRuleFile,
	  &lexicon_hash, &good_right_hash,
	  &good_left_hash, &seenTagging_hash,
	  &bigramArray, &lRuleArray, &cRuleArray);

  fclose(cRuleFile);
  fclose(lRuleFile);
  fclose(bigrams);
  fclose(lexicon);
}

POST::~POST ()
{
  int i;
  
  Registry_traverse(lexicon_hash,FreeRegistryEntry,NULL);
  Registry_destroy(lexicon_hash);

  Registry_traverse(good_right_hash,FreeRegistryEntry,NULL);
  Registry_destroy(good_right_hash);

  Registry_traverse(good_left_hash,FreeRegistryEntry,NULL);
  Registry_destroy(good_left_hash);

  Registry_traverse(seenTagging_hash,FreeRegistryEntry,NULL);
  Registry_destroy(seenTagging_hash);

  for (i=0;i<Darray_len(lRuleArray);++i)
    free((char*)Darray_get(lRuleArray,i));

  for (i=0;i<Darray_len(cRuleArray);++i)
    free((char*)Darray_get(cRuleArray,i));

  for (i=0;i<Darray_len(bigramArray);++i)
    free((char*)Darray_get(bigramArray,i));

  Darray_destroy(lRuleArray);
  Darray_destroy(cRuleArray);
  Darray_destroy(bigramArray);
}

void POST::run (list<string> &sList)
{
  int numSen=sList.size();
  list<string>::iterator si;

  if (numSen)
    {
      int i;
      char **buffer;

      buffer=new char*[numSen];
      for (si=sList.begin(),i=0;si!=sList.end();++si,++i)
	  {
	    // convert the string to a char array that the tagger in C
	    // code can handle
	    buffer[i] = new char[si->length()+1];
	    strcpy(buffer[i],si->c_str());
	  }

      // ================= Call the tagger now ================= 
      buffer = StartStateTagger (buffer, numSen, lexicon_hash,
				 good_right_hash, good_left_hash,
				 bigramArray, lRuleArray);

      // lexicon_hash is effectively the same as a word_hash, which
      // is read from the lexicon file and originally used here. We
      // just replace word_hash with lexicon_hash here (save memory!)
      buffer = FinalStateTagger (buffer, numSen, seenTagging_hash,
				 lexicon_hash, cRuleArray);
  
      // ============== End of call to the tagger ==============

      // update sList to be tags
      for(si=sList.begin(),i=0;si!=sList.end();++si,++i) (*si)=buffer[i];

      delete[] buffer;
    }
}
